C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   FUNCTION NAME     - INNERC                                                 +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           DOUBLE PRECISION FUNCTION:                                         +
C           COMPUTES THE INNER PRODUCT BETWEEN TWO VECTORS WHICH ARE           +
C           A ROW AND A COLUMN OF TWO ARRAYS.                                  +
C                                                                              +
C   USAGE:                                                                     +
C           INNERC (A, IA, B, IB, IR, IP, IFLAG)                               +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           A      - INPUT REAL MATRIX CONTAINING A CERTAIN NUMBER OF ROW      +
C                    VECTORS.                                                  +
C                                                                              +
C           IA     - INPUT ROW OF THE MATRIX A. FIRST VECTOR OF THE INNER      +
C                    PRODUCT.                                                  +
C                                                                              +
C           B      - INPUT REAL MATRIX OF DIMENSION CONTAINING A CERTAIN       +
C                    NUMBER OF COLUMN VECTORS.                                 +
C                                                                              +
C           IB     - INPUT COLUMN OF MATRIX B. SECOND VECTOR OF THE INNER      +
C                    PRODUCT.                                                  +
C                                                                              +
C           IR     - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR THE       +
C                    MATRICES A AND B.                                         +
C                                                                              +
C           IP     - NUMBER OF ELEMENTS TO BE CONSIDERED IN EACH VECTOR        +
C                    STARTING FROM THE FIRST ONE.                              +
C                                                                              +
C           IFLAG  - INPUT INTEGER FLAG WHICH INDICATES IF THE MATRIX B IS     +
C                    AN HESSENBERG MATRIX WITH A PARTICULAR STORAGE (IFLAG=1). +
C                    IF IFLAG = 0, B IS AN ARBITRARY MATRIX.                   +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C        - ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROGRAM MUST BE      +
C          IN DOUBLE PRECISION.                                                +
C        - IN THE CALLING PROCEDURE, THE ROWS DIMENSION FOR THE MATRICES A     +
C          AND B MUST BE THE SAME.                                             +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                       
      DOUBLE PRECISION FUNCTION INNERC (A, IA, B, IB, IR, IP, IFLAG)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      DOUBLE PRECISION A(1), B(1) 
      INTEGER IA, IB, IFLAG, IP, IR
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      DOUBLE PRECISION ACC
      INTEGER I, IBSTART
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (IFLAG .EQ. 0) THEN
         IBSTART = (IB - 1) * IR 
      ELSE
         IBSTART = (IB - 1) * (IB + 2) / 2
      END IF
      ACC  = 0.0D0
      DO 10 I = 1, IP
         ACC = ACC + A(IA + IR*(I-1)) * B(IBSTART+I)
   10 CONTINUE      
C
C                        ... SETS THE RESULT VALUE
C
      INNERC = ACC 
      RETURN
      END
