C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - INVERS                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           DETERMINES IF A MATRIX IS SINGULAR OR NOT.                         +
C           IF THE MATRIX IS NON-SINGULAR THE SUBROUTINE RETURNS THE           +
C           INVERSE MATRIX.                                                    +
C                                                                              +
C   USAGE:                                                                     +
C           CALL INVERS (A, N, IR, EPS, INDS)                                  +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           A      - INPUT REAL MATRIX OF DIMENSION (IR,2*IR).                 +
C                                                                              +
C           N      - INPUT INTEGER INDICATING THE DIMENSION OF THE A-ARRAY.    +
C                                                                              +
C           IR     - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR           +
C                    THE MATRIX A.                                             +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           INDS   - OUTPUT INTEGER VALUE.                                     +
C                    IF INDS = 0 THEN THE MATRIX A IS NON-SINGULAR.            +
C                    IF INDS = 1 THEN THE MATRIX A IS SINGULAR.                +
C                                                                              +
C   EXTERNAL MODULES:                                                          +
C                                                                              +
C       - SUBROUTINE     GAUSS                                                 +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C        - ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROGRAM MUST BE      +
C          IN DOUBLE PRECISION.                                                +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                       
      SUBROUTINE INVERS (A, N, IR, EPS, INDS)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER INDS, IR, N
      DOUBLE PRECISION EPS
      DOUBLE PRECISION A(IR,1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      INTEGER I, J
C
C                        ... EXECUTABLE STATEMENTS
C
C                        ... ADDS, TO THE GIVEN MATRIX, N COLUMN VECTORS
C                            WHICH FORM THE CANONICAL BASIS 
C
      DO 20 I = 1, N
         DO 10 J = N+1, N+N
            A(I,J) = 0.0D0
   10    CONTINUE
         A(I,N+I) = 1.0D0
   20 CONTINUE
C
C                        ... SOLVES THE SYSTEM WITH GAUSS METHOD. IF THE MATRIX
C                            A IS NON-SINGULAR THEN, IN THE SAME ARRAY A,
C                            (ROWS 1 TO N AND COLUMNS N+1 to 2*N) THERE IS,
C                            IN OUTPUT, THE INVERSE MATRIX. OTHERWISE INDS IS 
C                            SET TO 1 IN OUTPUT.
C
      CALL GAUSS (A, N, IR, EPS, INDS)
C
      IF (INDS .EQ. 0) THEN
C
C                        ... SUPERSEDES THE ORIGINAL MATRIX A WITH THE INVERSE 
C                            MATRIX.
C
         DO 40 I = 1, N
            DO 30 J = 1, N
               A(I,J) = A(I,N+J)
   30       CONTINUE
   40    CONTINUE
      END IF
      RETURN
      END
