C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   PROGRAM NAME     - MINVLAP                                                 +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           MAIN PROGRAM TO TEST THE SUBROUTINE INVLAP.                        +
C                                                                              +
C   MODULES REQUIRED:                                                          +
C                                                                              +
C       -  SUBROUTINE   INVLAP                                                 +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
      PROGRAM MINVLAP
C
C                        ... SPECIFICATIONS FOR VARIABLES AND CONSTANTS
C
      INTEGER I, IER, J, KDEG, NV, NPT
      DOUBLE PRECISION A, FF, FAC, FT, H, EPS, T
C
C                        ... SPECIFICATIONS FOR PARAMETER CONSTANTS
C
      PARAMETER (NV=15, KDEG=3, EPS=1.0D-30)
C
C                        ... SPECIFICATIONS FOR VECTORS AND MATRICES
C                            REMARK: THE DIMENSION OF WK MUST BE
C                            (0:MAX0(8*KDEG+13,3*KDEG+NV+2))
C 
      DOUBLE PRECISION P(0:2*KDEG+1), F(0:2*KDEG+1), V(0:NV),
     *       WK(0:8*KDEG+13)
C
C                        ... EXECUTABLE STATEMENTS
C
      WRITE (*,*) ' OUTPUT RESULTS FROM MINVLAP'
C
      NPT = 2 * KDEG + 1
      H   = 2.0D0 / NPT
      A   = 1.0D0
C
      WRITE (*,*) 
      WRITE (*,*) ' DEGREE OF THE NUMERATOR  =', KDEG
      WRITE (*,*) ' NUMBER OF POINTS         =', NPT+1
      WRITE (*,*) ' MAX DEGREE OF THE SERIES =', NV
C
C                        ... COMPUTES THE VALUES F(p ) AT THE POINTS p
C                                                   n                 n
      DO 10 I = 0, NPT
         P(I) = 0.1D0 + I * H
         F(I) = DLOG (1.0D0 + A**2 / P(I)**2)
   10 CONTINUE
C
C                        ... CALL INVLAP
C
      CALL INVLAP (KDEG, P, F, EPS, NV, V, WK, IER)
C
      IF (IER .NE. 0) THEN
         WRITE (*,*) ' STOP DUE TO IER =',IER
         STOP
      END IF
C
C                        ... COMPUTES THE VALUES f(t ) FOR j = 1, ..., 5
C                                                   j
C
      DO 30 J = 1, 5
         T   = -1.0D0 + (J-1) * 0.4D0
C
C                        ... COMPUTES THE EXACT RESULT
C                                                  
         FT  = 2.0D0 * (1.0D0 - DCOS(A*T)) /T
C
         WRITE (*,100) T, FT
C
C                        ... COMPUTES THE APPROXIMATE VALUES
C                                                  
         FF  = V(0)
         FAC = 1.0D0
         DO 20 I = 1, NV
            FAC = FAC * T / I
            FF  = FF + V(I) * FAC
            WRITE (*,200) I+1, FF
   20    CONTINUE
   30 CONTINUE
      STOP
  100 FORMAT(/T3,' T ',F9.2, T27, ' EXACT VALUE ',D26.16//
     *        T3,' N. OF TERMS', T25, ' APPROX. VALUE ')
  200 FORMAT(I12,D26.16)
      END
