C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - RCPROD                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           COMPUTES THE PRODUCT OF TWO MATRICES.                              +
C                                                                              +
C   USAGE:                                                                     +
C           CALL RCPROD (A, IARST, IACST, IRA, B, IBRST, IBCST, IRB,           +
C                        C, ICRST, ICCST, IRC, NAROW, NACOL, NBCOL,            +
C                        C1COEF, C2COEF, IFLAG, WK)                            +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           A      - INPUT REAL MATRIX OF DIMENSION (IRA,IRB).                 +
C                                                                              +
C           IARST  - INPUT INDEX OF THE STARTING ROW OF THE MATRIX A.          +
C                                                                              +
C           IACST  - INPUT INDEX OF THE STARTING COLUMN OF THE MATRIX A.       +
C                                                                              +
C           IRA    - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR           +
C                    THE MATRIX A.                                             +
C                                                                              +
C           B      - INPUT REAL MATRIX OF DIMENSION (IRB,*).                   +
C                                                                              +
C           IBRST  - INPUT INDEX OF THE STARTING ROW OF THE MATRIX B.          +
C                                                                              +
C           IBCST  - INPUT INDEX OF THE STARTING COLUMN OF THE MATRIX B.       +
C                                                                              +
C           IRB    - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR           +
C                    THE MATRIX B.                                             +
C                                                                              +
C           C      - OUTPUT REAL PRODUCT MATRIX OF DIMENSION (IRA,*).          +
C                                                                              +
C           ICRST  - INPUT INDEX OF THE STARTING ROW OF THE MATRIX C.          +
C                                                                              +
C           ICCST  - INPUT INDEX OF THE STARTING COLUMN OF THE MATRIX C.       +
C                                                                              +
C           IRC    - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR           +
C                    THE MATRIX C.                                             +
C                                                                              +
C           NAROW  - INPUT NUMBER OF ROWS OF THE MATRIX A TO BE CONSIDERED.    +
C                                                                              +
C           NACOL  - INPUT NUMBER OF COLUMNS OF THE MATRIX A AND NUMBER OF     +
C                    ROWS OF THE MATRIX B TO BE CONSIDERED.                    +
C                                                                              +
C           NBCOL  - INPUT NUMBER OF COLUMNS OF THE MATRIX B TO BE CONSIDERED. +
C                                                                              +
C           C1COEF - INPUT REAL NUMBER.                                        +
C                    IF C1COEF = 0.0D0 THEN THE PRODUCT MATRIX IS STORED IN    +
C                    THE C-ARRAY.                                              +
C                    IF C1COEF NOT EQUAL TO ZERO, THEN THE PREVIOUS C-ARRAY    +
C                    IS MULTIPLIED BY C1COEF AND ADDED TO THE PRODUCT MATRIX.  +
C                                                                              +
C           C2COEF - INPUT REAL NUMBER.                                        +
C                    THE PRODUCT MATRIX IS ALWAYS MULTYPLIED BY C2COEF. THUS   +
C                    THE VALUE OF C2COEF MUST BE SET TO 1.0D0 IN THE NORMAL    +
C                    USE OF THE SUBROUTINE.                                    +
C                                                                              +
C           IFLAG  - IF IFLAG = 0 THEN THE RESULTANT C-ARRAY IS NOT COINCIDENT +
C                                 WITH THE B-ARRAY, BUT IT MAY COINCIDE WITH   +
C                                 THE A-ARRAY.                                 +
C                    IF IFLAG NOT EQUAL TO ZERO THEN THE RESULTANT C-ARRAY     +
C                                 COINCIDES WITH THE B-ARRAY.                  +
C                                                                              +
C           WK     - REAL WORKING VECTOR. WK HAS TO BE DIMENSIONED AT LEAST    +
C                    TO THE GREATEST DIMENSION BETWEEN A AND B.                +
C                                                                              +
C   EXTERNAL MODULE:                                                           +
C                                                                              +
C       -  DOUBLE PRECISION FUNCTION  INNERG                                   +
C                                                                              +
C   REMARKS:                                                                   +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROGRAM MUST BE      +
C          IN DOUBLE PRECISION.                                                +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                       
      SUBROUTINE RCPROD (A, IARST, IACST, IRA, B, IBRST, IBCST, IRB, 
     *                   C, ICRST, ICCST, IRC, NAROW, NACOL, NBCOL, 
     *                   C1COEF, C2COEF, IFLAG, WK)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IARST, IACST, IRA, IBRST, IBCST, IRB, ICRST, ICCST, IRC, 
     *        NAROW, NACOL, NBCOL, IFLAG
      DOUBLE PRECISION C1COEF, C2COEF
      DOUBLE PRECISION A(IRA,1), B(IRB,1), C(IRC,1), WK(1)
C
C                        ... SPECIFICATIONS FOR EXTERNAL FUNCTIONS
C
      DOUBLE PRECISION INNERG
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      INTEGER I, J, IASTART, IBSTART, INDCROW, INDCCOL
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (IFLAG .EQ. 0 ) THEN
         DO 30 I = 1, NAROW 
            IASTART = IARST + I - 1
            INDCROW = ICRST + I - 1
            DO 10 J = 1, NBCOL
               IBSTART = IBCST + J - 1
               WK(J) = INNERG (A, IASTART, IACST, IRA, 
     *                         B, IBRST, IBSTART, IRB, NACOL)
   10       CONTINUE
            DO 20 J = 1, NBCOL
               INDCCOL = ICCST + J - 1
               C(INDCROW, INDCCOL) = C1COEF * C(INDCROW, INDCCOL)
     *                               + C2COEF * WK(J)
   20       CONTINUE
   30    CONTINUE
      ELSE
         DO 60 J = 1, NBCOL
            IBSTART = IBCST + J - 1
            INDCCOL = ICCST + J - 1
            DO 40 I = 1, NAROW
               IASTART = IARST + I - 1
               WK(I) = INNERG (A, IASTART, IACST, IRA, 
     *                         B, IBRST, IBSTART, IRB, NACOL)
   40       CONTINUE
            DO 50 I = 1, NAROW
               INDCROW = ICRST + I - 1
               C(INDCROW, INDCCOL) = C1COEF * C(INDCROW, INDCCOL)
     *                               + C2COEF * WK(I)
   50       CONTINUE
   60    CONTINUE
      END IF
      RETURN
      END
