C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - VEALGO                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           VECTOR SEQUENCE EXTRAPOLATION VIA THE E-ALGORITHM                  +
C                                                                              +
C   USAGE:                                                                     +
C           CALL VEALGO (E, MAXCOL, INIT, EPS, EINIT, GINIT, Y, IR,            +
C              IP, G, WG, IER)                                                 +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           E      - OUTPUT REAL MATRIX. IN THE CALLING PROCEDURE THE          +
C                    DIMENSIONS OF E MUST BE (IP,0:IE), WITH IE GREATER OR     +
C                    EQUAL TO MAXCOL. THIS MATRIX CONTAINS, AFTER THE k-TH     +
C                    CALL THE LAST COMPUTED ROW OF THE E-ARRAY:                +
C                                                                              +
C                                          (k-1)  (k-2)       (0)              +
C                     - IF k <= MAXCOL+1  E    , E     ,..., E                 +
C                                          0      1           k-1              +
C                                                                              +
C                                          (k-1)  (k-2)       (k-1-MAXCOL)     +
C                     - IF k > MAXCOL+1   E    , E     ,..., E                 +
C                                          0      1           MAXCOL           +
C                                                                              +
C           MAXCOL - INPUT INTEGER GIVING THE INDEX OF THE LAST COLUMN         +
C                    OF THE E-ARRAY THAT THE USER WANTS TO COMPUTE.            +
C                                         (n)                                  +
C                    (THE LOWER INDEX OF E   ).                                +
C                                         k                                    +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY      +
C                    THE SUBROUTINE DURING THE FIRST CALL.                     +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           EINIT  - INPUT/OUTPUT REAL VECTOR. IT MUST CONTAIN THE VALUE       +
C                    OF S    BEFORE THE k-TH CALL OF THE SUBROUTINE.           +
C                        k-1                                                   +
C                    IT CONTAINS AFTER THE k-TH CALL OF THE SUBROUTINE         +
C                                                                              +
C                     (0)                                                      +
C                    E            IF  k <= MAXCOL+1                            +
C                     k-1                                                      +
C                                                                              +
C                     (k-1-MAXCOL)                                             +
C                    E            IF  k >  MAXCOL+1                            +
C                     MAXCOL                                                   +
C                                                                              +
C                    THE SUBROUTINE RETURNS THE SEQUENCE                       +
C                                                                              +
C                     (0)   (0)   (0)         (0)      (1)      (2)            +
C                    E   , E   , E   , ... , E      , E      , E      , ...    +
C                     0     1     2           MAXCOL   MAXCOL   MAXCOL         +
C                                                                              +
C           GINIT  - INPUT REAL MATRIX. IN THE CALLING PROCEDURE THE SECOND    +
C                    DIMENSION OF GINIT MUST BE GREATER OR EQUAL TO 2*MAXCOL.  +
C                    BEFORE THE k-TH CALL OF THE SUBROUTINE IT MUST CONTAIN    +
C                    IF k = 1   GINIT(1)= G (0)                                +
C                                          1                                   +
C                                                                              +
C                    IF k = 2   GINIT(1)= G (1)                                +
C                                          1                                   +
C                                                                              +
C                    IF 3 <= k <= MAXCOL+1                                     +
C                                                                              +
C                       GINIT=[G (k-1), G (k-1), ... , G   (k-1),              +
C                               1        2              k-2                    +
C                                                                              +
C                              G   (0), G   (1), ... , G   (k-1)]              +
C                               k-1      k-1            k-1                    +
C                                                                              +
C                    IF k > MAXCOL+1                                           +
C                                                                              +
C                       GINIT=[G (k-1), G (k-1), ... , G   (k-1)]              +
C                               1        2              MAXCOL                 +
C                                                                              +
C           Y      - INPUT REAL VECTOR WHICH MUST CONTAIN THE ARBITRARY        +
C                    VECTOR y.                                                 +
C                                                                              +
C           IR     - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR THE       +
C                    MATRICES E, GINIT, G, AND FOR THE VECTORS EINIT, WG       +
C                    AND Y.                                                    +
C                                                                              +
C           IP     - INPUT DIMENSION OF THE VECTOR SPACE (NUMBER OF            +
C                    ROWS FOR THE MATRICES E, GINIT, G, AND FOR THE            +
C                    VECTORS EINIT, WG AND Y).                                 +
C                                                                              +
C           G      - WORKING REAL MATRIX IN WHICH THE LAST ROW AND THE         +
C                    LAST COLUMN OF EACH G  ARRAY ARE SAVED FOR THE NEXT       +
C                                         i                                    +
C                    CALL OF THE SUBROUTINE. IN THE CALLING PROCEDURE THE      +
C                    SECOND DIMENSION OF G MUST BE GREATER OR EQUAL TO         +
C                    MAXCOL*(MAXCOL+1) AND TO MAXCOL*NUMBER_OF_CALLS.          +
C                                                                              +
C           WG     - WORKING REAL VECTOR OF DIMENSION IR.                      +
C                                                                              +
C           IER    - OUTPUT INDEX WARNING/ERROR.                               +
C                    IER = 100     DIVISION BY ZERO IN THE SUBROUTINE.         +
C                    IER = 200     CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                    IER = 300     ROWS DIMENSION TOO SMALL IN THE CALLING     +
C                                  PROCEDURE COMPARED TO THE VECTOR SPACE      +
C                                  DIMENSION.                                  +
C                                                                              +
C                                                                              +
C   EXTERNAL MODULES:                                                          +
C                                                                              +
C       -  DOUBLE PRECISION FUNCTION  INNERP                                   +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST       +
C          BE IN DOUBLE PRECISION. THE VECTOR WS AND THE MATRICES E AND G      +
C          MUST NOT BE MODIFIED BY THE USER BETWEEN TWO CONSECUTIVE CALLS      +
C          OF THE SUBROUTINE.                                                  +
C                                  (0)                                         +
C       -  TO OBTAIN THE VALUE OF E   (J <= MAXCOL), THE USER MUST CALL THIS   +
C                                  J                                           +
C          SUBROUTINE (J+1) TIMES.                                             +
C       -  IN THE CALLING PROCEDURE, THE ROWS DIMENSION FOR THE MATRICES E,    +
C          GINIT, G, AND THE DIMENSIONS OF THE VECTORS EINIT, WG, Y, MUST      +
C          BE THE SAME.                                                        +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE             +
C          MATRIX AND VECTOR ARGUMENTS (IR=IP) ARE:                            +
C           E(IP,0:MAXCOL)                                                     +
C           EINIT(IP)                                                          +
C           GINIT(IP,2*MAXCOL)                                                 +
C           Y(IP)                                                              +
C           G(IP,MAXDIM),MAXDIM=MAX(MAXCOL*(MAXCOL+1),MAXCOL*NUMBER_OF_CALLS)  +
C           WG(IP)                                                             +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                       
      SUBROUTINE VEALGO (E, MAXCOL, INIT, EPS, EINIT, GINIT, Y, IR, 
     *       IP, G, WG, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, INIT, IP, IR, MAXCOL
      DOUBLE PRECISION EPS
      DOUBLE PRECISION E(IR,0:1), EINIT(1), G(IR,1), GINIT(IR,1), Y(1),
     *  WG(1)
C
C                        ... SPECIFICATIONS FOR EXTERNAL FUNCTIONS
C
      DOUBLE PRECISION INNERP
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      DOUBLE PRECISION R, RD
      INTEGER I, II, IND, INDI, INDI1, INDJ, INDM, J
      SAVE INDM
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         IF (IP .GT. IR) THEN
            IER = 300
            RETURN
         END IF
         IER    = 0                                                           
         INDM   = 0
         INIT   = 1
         DO 10 II = 1, IP
            E(II,0) = EINIT(II)
            G(II,1) = GINIT(II,1)
   10    CONTINUE
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE               
C
      IF (IER .NE. 0) THEN
         IER = 200
         RETURN
      END IF
C
      IF (MAXCOL .EQ. 0) THEN
         DO 20 II = 1, IP
            E(II,0) = EINIT(II)
   20    CONTINUE
         RETURN
      END IF
C
C                        ... INDM REPRESENTS THE (NUMBER_OF_CALLS -1)
C
      INDM = INDM + 1
C
C                        ... ADD AN ELEMENT TO G  ARRAY AT EACH CALL
C                                               1
C
      DO 30 II = 1, IP
         G(II,1+INDM*MAXCOL) = GINIT(II,1)
   30 CONTINUE
      IF (INDM .NE. 1) THEN
C
C                        ... COMPUTE AND SAVE IN G THE NECESSARY ELEMENTS 
C                            OF THE G ARRAYS (ONLY FROM THE THIRD CALL)
C                                    i
C
         IF (INDM .GE. 3) THEN
C
C                        ... COMPUTE THE NEXT ROW OF ALL THE PREVIOUS G  
C                                                                      i
C                            ARRAYS (i = 2 TO INDM-1 IF INDM <= MAXCOL+1)    
C                                   (i = 2 TO MAXCOL IF INDM > MAXCOL+1)
C                            AND SAVE IN G THE NEW COMPUTED ELEMENTS 
C
            IND = MIN0(INDM-1,MAXCOL)
            DO 80 I = 2, IND
               DO 70 J = 1, I-1
                  INDI = I + (J-1) * MAXCOL
                  INDJ = J + INDM * MAXCOL
                  DO 40 II = 1, IP
                     WG(II) = G(II,INDJ-MAXCOL) - G(II,INDJ)
   40             CONTINUE
                  RD = INNERP(Y, 1, WG, 1, IR, IP)
                  IF (DABS(RD) .LT. EPS) THEN
                     IER = 100
                     RETURN
                  END IF
                  DO 50 II = 1, IP
                     WG(II) = G(II,INDI) - GINIT(II,I)
   50             CONTINUE
                  RD = INNERP(Y, 1, WG, 1, IR, IP)/RD
                  DO 60 II = 1, IP  
                     R = GINIT(II,I) - RD * G(II,INDJ)
                     G(II,INDI)    = GINIT(II,I)
                     GINIT(II,I) = R
                     IF (J .EQ. (I-1)) G(II,I+INDM*MAXCOL) = R
   60             CONTINUE
   70          CONTINUE
   80       CONTINUE
         END IF
         IF (INDM .LE. MAXCOL) THEN
C
C                        ... COMPUTE THE NEW G     ARRAY  IF INDM <= MAXCOL
C                                             INDM
C                            AND SAVE IN G THE RELATIVE LAST ROW AND COLUMN
C
            DO 90 II = 1, IP
               G(II,INDM) = GINIT(II,INDM)
   90       CONTINUE
            DO 140 I= 1, INDM
               INDI1 = INDM + I
               IND   = MIN0(I,INDM-1)
               DO 130 J = 1, IND
                  INDI = J + I*MAXCOL
                  INDJ = INDM + (J-1)*MAXCOL
                  DO 100 II = 1, IP
                     WG(II) = G(II,INDI-MAXCOL) - G(II,INDI)
  100             CONTINUE
                  RD = INNERP(Y, 1, WG, 1, IR, IP)
                  IF (DABS(RD) .LT. EPS) THEN
                     IER = 100
                     RETURN
                  END IF
                  DO 110 II = 1, IP
                     WG(II) = G(II,INDJ) - GINIT(II,INDI1)
  110             CONTINUE
                  RD = INNERP(Y, 1, WG, 1, IR, IP)/RD
                  DO 120 II = 1, IP
                     R = GINIT(II,INDI1) - RD * G(II,INDI)
                     G(II,INDJ)      = GINIT(II,INDI1)
                     GINIT(II,INDI1) = R
                     IF (J .EQ. IND) G(II,INDM+I*MAXCOL) = R
  120             CONTINUE
  130          CONTINUE
  140       CONTINUE
         END IF
      END IF
C
C                        ... COMPUTE THE NEW ELEMENTS OF THE NEXT ROW
C                            IN THE E ARRAY  
C
      IND = MIN0(INDM,MAXCOL)
      DO 180 I = 1, IND
         INDI = I + INDM*MAXCOL
         DO 150 II = 1, IP
            WG(II) = G(II,INDI-MAXCOL) - G(II,INDI) 
  150    CONTINUE
         RD = INNERP(Y, 1, WG, 1, IR, IP)
         IF (DABS(RD) .LT. EPS) THEN
            IER = 100
            RETURN
         END IF
         DO 160 II = 1, IP
            WG(II) = E(II,I-1) - EINIT(II)
  160    CONTINUE
         RD = INNERP(Y, 1, WG, 1, IR, IP)/RD
         DO 170 II = 1, IP
            R         = EINIT(II) - RD * G(II,INDI)
            E(II,I-1) = EINIT(II)
            EINIT(II) = R
            IF (I .EQ. IND) E(II,IND) = R
  170    CONTINUE
C
C                        ... END LOOP FOR I.
C
  180 CONTINUE
      RETURN
      END
